#############################################################################
## Ruby-based augmentation of ASFinder extension classes.
##
## --------------------------------------------------------------------------
## Copyright (C) 2008,2012 The Regents of the University of California.
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## $Id: asfinder.rb,v 1.5 2012/08/07 19:34:00 youngh Exp $
#############################################################################

require "asfinderext"

module CAIDA

class ASFinder

  alias_method :__raw_initialize, :initialize

  def initialize(path)
    if path == nil || !path.kind_of?(String)
      raise ArgumentError, "invalid path argument"
    end

    command = nil
    case path
    when /\.gz$/ then command = "gzip -dc"
    when /\.bz2$/ then command = "bzip2 -dc"
    when /\.lzo$/ then command = "lzop -dc"
    end

    retval = nil
    if command
      ENV['RBASFINDER_FILE_OPEN_PATH'] = path
      io = IO.popen "#{command} \"$RBASFINDER_FILE_OPEN_PATH\""
      begin
        fd_path = "/dev/fd/" + io.fileno.to_s
        retval = __raw_initialize fd_path
      ensure
        if retval
          instance_variable_set :@___popen_io, io
        else
          io.close rescue nil
        end
      end
    else
      retval = __raw_initialize path
    end
    retval
  end

  def close
    io = instance_variable_get :@___popen_io
    if io
      io.close rescue nil
      instance_variable_set :@___popen_io, nil
    end
  end

end


#############################################################################

class ASFinder6

  alias_method :__raw_initialize, :initialize

  def initialize(path)
    if path == nil || !path.kind_of?(String)
      raise ArgumentError, "invalid path argument"
    end

    command = nil
    case path
    when /\.gz$/ then command = "gzip -dc"
    when /\.bz2$/ then command = "bzip2 -dc"
    when /\.lzo$/ then command = "lzop -dc"
    end

    retval = nil
    if command
      ENV['RBASFINDER_FILE_OPEN_PATH'] = path
      io = IO.popen "#{command} \"$RBASFINDER_FILE_OPEN_PATH\""
      begin
        fd_path = "/dev/fd/" + io.fileno.to_s
        retval = __raw_initialize fd_path
      ensure
        if retval
          instance_variable_set :@___popen_io, io
        else
          io.close rescue nil
        end
      end
    else
      retval = __raw_initialize path
    end
    retval
  end

  def close
    io = instance_variable_get :@___popen_io
    if io
      io.close rescue nil
      instance_variable_set :@___popen_io, nil
    end
  end

end

end # module CAIDA
